#include <stdio.h>
#include <dinput.h>

#include "tankwars.h"

// Direct Input
LPDIRECTINPUT7			g_DI;
// Keyboard Device
LPDIRECTINPUTDEVICE7	g_KDIDev;

HDC	hDC=NULL;			// Private GDI Device Context
HGLRC hRC=NULL;			// Permanent Rendering Context
HWND hWnd=NULL;			// Holds Our Window Handle
HINSTANCE hInstance;	// Holds The Instance Of The Application

BYTE key[256];			// Array Used For The Keyboard Routine
bool mousebutton[2];	// Mouse buttons
bool active=TRUE;		// Window Active Flag Set To TRUE By Default
bool fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default

extern int screen_h;
extern int screen_w;

GLuint	base;			// Base Display List For The Font
GLuint	texture[TOTAL_TEXTURES];

// Declaration For WndProc:
LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

int gamestart(void);
int InitGL(GLvoid);

// Loads A Bitmap Image:
AUX_RGBImageRec *LoadBMP(char *Filename)
{
	FILE *File=NULL;
	// Make Sure A Filename Was Given:
	if (!Filename)
		return NULL;
	File=fopen(Filename,"r");
	
	// Does The File Exist?
	if (File)
	{
		// Close The Handle:
		fclose(File);
		// Load The Bitmap And Return A Pointer:
		return auxDIBImageLoad(Filename);
	}
	// If Load Failed Return NULL:
	return NULL;
}

// Build Our Font Display List:
GLvoid BuildFont(GLvoid)
{
	int loop;

	float	cx;
	float	cy;
	
	// Creating 256 Display Lists:
	base=glGenLists(256);
	
	// Select Our Font Texture:
	glBindTexture(GL_TEXTURE_2D, texture[FONT_TEXTURE]);
	for (loop=0; loop<256; loop++)
	{
		cx=float(loop%16)/16.0f;
		cy=float(loop/16)/16.0f;
		// Start Building A List:
		glNewList(base+loop,GL_COMPILE);
			glBegin(GL_QUADS);					// Use A Quad For Each Character
				glTexCoord2f(cx,1-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0,0);				// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1-cy-0.0625f);		// Texture Coord (Bottom Right)
				glVertex2i(16,0);				// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1-cy);			// Texture Coord (Top Right)
				glVertex2i(16,16);				// Vertex Coord (Top Right)
				glTexCoord2f(cx,1-cy);				// Texture Coord (Top Left)
				glVertex2i(0,16);				// Vertex Coord (Top Left)
			glEnd();						// Done Building Our Quad (Character)
			glTranslated(10,0,0);					// Move To The Right Of The Character
		glEndList();							// Done Building The Display List
	}									// Loop Until All 256 Are Built
}

GLvoid KillFont(GLvoid)								// Delete The Font From Memory
{
	glDeleteLists(base,256);						// Delete All 256 Display Lists
}

// Where The Printing Happens:
GLvoid glPrint(GLint x, GLint y, int set, const char *fmt, ...)
{
	char string[256];
	va_list		ap;					// Pointer To List Of Arguments
	va_start(ap, fmt);					// Parses The String For Variables
	    vsprintf(string, fmt, ap);				// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text


	if (set>1)								// Is set Greater Than One?
		set=1;								// If So, Make Set Equal One
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texture[FONT_TEXTURE]);	// Select Our Font Texture
	glDisable(GL_DEPTH_TEST);						// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();								// Store The Projection Matrix
	glLoadIdentity();							// Reset The Projection Matrix
	glOrtho(0,640,0,480,-1,1);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);						// Select The Modelview Matrix
	glPushMatrix();								// Store The Modelview Matrix
	glLoadIdentity();							// Reset The Modelview Matrix
	glTranslated(x,y,0);							// Position The Text (0,0 - Bottom Left)
	glListBase(base-32+(128*set));						// Choose The Font Set (0 or 1)
	glCallLists(strlen(string),GL_BYTE,string);				// Write The Text To The Screen
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();								// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);						// Select The Modelview Matrix
	glPopMatrix();								// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);						// Enables Depth Testing
	glDisable(GL_BLEND);
}



// This function resizes and initialize the GL window
GLvoid ReSizeGLScene(GLsizei width, GLsizei height)
{
	// Prevent A Divide By Zero By:
	if (height==0)
		height=1;
	
	// Reset The Current Viewport:
	glViewport(0,0,width,height);

	// Select The Projection Matrix:
	glMatrixMode(GL_PROJECTION);
	// Reset The Projection Matrix:
	glLoadIdentity();

	// Calculate The Aspect Ratio Of The Window:
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,260.0f,430.0f);

	// Select The Modelview Matrix:
	glMatrixMode(GL_MODELVIEW);
	// Reset The Modelview Matrix:
	glLoadIdentity();
}

// This function Properly Kills The Window:
GLvoid KillGLWindow(GLvoid)
{
	// Are We In Fullscreen Mode?
	if (fullscreen)
	{
		// Switch Back To The Desktop:
		ChangeDisplaySettings(NULL,0);
		// Show Mouse Pointer
		ShowCursor(TRUE);
	}

	// Do We Have A Rendering Context?
	if (hRC)
	{
		// Are We Able To Release The DC And RC Contexts?
		if (!wglMakeCurrent(NULL,NULL))
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);

		// Are We Able To Delete The RC?
		if (!wglDeleteContext(hRC))
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		
		hRC=NULL;
	}

	// Are We Able To Release The DC?
	if (hDC && !ReleaseDC(hWnd,hDC))
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;
	}

	// Are We Able To Destroy The Window?
	if (hWnd && !DestroyWindow(hWnd))
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;
	}

	// Are We Able To Unregister Class?
	if (!UnregisterClass("OpenGL",hInstance))
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;
	}
	// Destroy The Font:
	KillFont();
}

// This Code Creates Our OpenGL Window.  Parameters Are:
// title			- Title To Appear At The Top Of The Window
// width			- Width Of The GL Window Or Fullscreen Mode
// height			- Height Of The GL Window Or Fullscreen Mode
// bits			- Number Of Bits To Use For Color (8/16/24/32)
// fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)
 BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style
	RECT		WindowRect;				// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;			// Set Left Value To 0
	WindowRect.right=(long)width;		// Set Right Value To Requested Width
	WindowRect.top=(long)0;				// Set Top Value To 0
	WindowRect.bottom=(long)height;		// Set Bottom Value To Requested Height

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	
	// Attempt Fullscreen Mode?
	if (fullscreen)
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;
			}
		}
	}

	// Are We Still In Fullscreen Mode?
	if (fullscreen)
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP;										// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;							// Windows Style
	}

	// Adjust Window To True Requested Size:
	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Extended Style For The Window
								"OpenGL",							// Class Name
								title,								// Window Title
								dwStyle |							// Defined Window Style
								WS_CLIPSIBLINGS |					// Required Window Style
								WS_CLIPCHILDREN,					// Required Window Style
								0, 0,								// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Dont Pass Anything To WM_CREATE
	{
		// Reset The Display:
		KillGLWindow();
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	return TRUE;
}

// Initializes Direct Input ( Add )
int DI_Init()
{
	// Create Direct Input
	if ( DirectInputCreateEx(	hInstance,			// Window Instance
					DIRECTINPUT_VERSION,		// Direct Input Version
					IID_IDirectInput7,		// Version 7
					(void**)&g_DI,			// Direct Input
					NULL ) )			// NULL Parameter
	{
		return(false);						// Couldn't Initialize Direct Input
	}

	// Create The Keyboard Device
	if ( g_DI->CreateDeviceEx(	GUID_SysKeyboard,		// Define Which Device Tto Create (KeyBoard,Mouse,or Joystick)
					IID_IDirectInputDevice7,	// Version 7
					(void**)&g_KDIDev,		// KeyBoard Device
					NULL ) )			// NULL Parameter
	{
		return(false);						// Couldn't Create The Keyboard Device
	}

	// Set The Keyboard Data Format
	if ( g_KDIDev->SetDataFormat(&c_dfDIKeyboard) )
	{
		return(false);						// Could Not Set The Data Format
	}

	// Set The Cooperative Level
	if ( g_KDIDev->SetCooperativeLevel(hWnd, DISCL_FOREGROUND | DISCL_EXCLUSIVE) )
	{
		return(false);						// Could Not Set The Cooperative Level
	}

	if (g_KDIDev)							// Did We Create The Keyboard Device?
		g_KDIDev->Acquire();					// If So, Acquire It
	else								// If Not
		return(false);						// Return False

	return(true);							// Everything Ok, Return True
}

// Destroys DX ( Add )
void DX_End()
{
	if (g_DI)
	{
		if (g_KDIDev)
		{
			g_KDIDev->Unacquire();
			g_KDIDev->Release();
			g_KDIDev = NULL;
		}

		g_DI->Release();
		g_DI = NULL;
	}
}


LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

/*
		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			key[wParam] = TRUE;						// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			key[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}
*/
		case WM_LBUTTONDOWN:
		{
			mousebutton[0]=1;
			return 0;
		}
		case WM_LBUTTONUP:
		{
			mousebutton[0]=0;
			return 0;
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	return gamestart();
}
